# ----------------------------------------------------------------------------
# Python shell dialog.  Displays output from python interpretter and accepts
# user typed commands.
#
import string
import sys
import traceback

import sparky
import sputil
import tkutil

# ----------------------------------------------------------------------------
#
class python_shell_dialog(tkutil.Dialog):

  def __init__(self):

    self.preferences = shell_preferences_dialog()
    
    tkutil.Dialog.__init__(self, 'Python Shell')

    w = tkutil.scrolling_text(self.top, 10, 80)
    w.frame.pack(side = 'top', fill = 'both', expand = 1)
    w.text.bind('<KeyRelease-Return>', self.enter_cb)
    w.text.focus_set()  # Initial focus child is text window
    self.text = w.text
    br = tkutil.button_row(self.top,
                           ('Preferences', self.preferences_cb),
                           ('Close', self.close_cb),
                           ('Help', sputil.help_cb('PythonShell')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

    #
    # Remember the original output streams before replacing them
    #
    sys.raw_stdout = sys.stdout
    sys.raw_stderr = sys.stderr

    #
    # Python output goes to this dialog
    #
    class output_object:
      def __init__(self, write_cb):
        self.write = write_cb

    #
    # Make stdout and stderr output go to this shell window
    #
    sys.stdout = output_object(self.output_text)
    sys.stderr = output_object(self.output_error)

    sys.stdout.write(sys.ps1)

  # ---------------------------------------------------------------------------
  # Note that last text character is always a new line, and 'end' refers
  # to start of next line.
  #
  def output_text(self, text):

    self.text.insert('end', text)
    self.text.mark_set('start_command', 'end - 1 chars')
    self.text.mark_gravity('start_command', 'left')
    self.text.see('end')

  # ---------------------------------------------------------------------------
  #
  def output_error(self, text):

    if self.preferences.raise_on_error():
      self.show_window(1)
    self.output_text(text)

  # ---------------------------------------------------------------------------
  #
  def preferences_cb(self):

    self.preferences.show_window(1)

  # ---------------------------------------------------------------------------
  #
  def enter_cb(self, event):

    if self.text.index('insert') == self.text.index('end - 1 chars'):
      cmd = self.text.get('start_command', 'end - 1 chars')
      command = remove_prompts(cmd)
      if is_complete_command(command):
        self.execute_command(command);
      else:
        self.text.insert('end', sys.ps2)

  # ---------------------------------------------------------------------------
  #
  def send_command(self, command):

    cmd = command + '\n'
    sys.stdout.write(cmd)
    self.execute_command(cmd)
    self.top.update()
  
  # ---------------------------------------------------------------------------
  #
  def execute_command(self, command):

    cmd = string.lstrip(command)
    if cmd:
      dict = sys.modules['__main__'].__dict__
      try:
        c = compile(cmd, '<stdin>', 'single')
        result = eval(c, dict, dict)
      except:
        traceback.print_exc()
      else:
        if result != None:
          sys.stdout.write(str(result) + '\n')

    sys.stdout.write(sys.ps1)

# ----------------------------------------------------------------------------
#
class shell_preferences_dialog(tkutil.Dialog):

  def __init__(self):

    tkutil.Dialog.__init__(self, 'Python Shell Preferences')

    cb = tkutil.checkbutton(self.top, 'Show Python shell if error occurs?', 1)
    cb.button.pack(side = 'top', anchor = 'nw')
    self.raise_variable = cb.variable

    br = tkutil.button_row(self.top,
                           ('Close', self.close_cb),
                           ('Help', sputil.help_cb('PythonShell')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def raise_on_error(self):

    return self.raise_variable.get()

# -----------------------------------------------------------------------------
#
def is_complete_command(command):

  return not is_compound_statement(command) or command[-2:] == '\n\n'

# -----------------------------------------------------------------------------
#
def is_compound_statement(command):

  return (initial_match("if ", command) or
          initial_match("while ", command) or
          initial_match("for ", command) or
          initial_match("try ", command) or
          initial_match("def ", command) or
          initial_match("class ", command))

# ---------------------------------------------------------------------------
#
def remove_prompts(command):

  lines = string.split(command, '\n')
  cmd = remove_initial_match(lines[0], sys.ps1)
  for line in lines[1:]:
    cmd = cmd + '\n' + remove_initial_match(line, sys.ps2)
  return cmd

# ----------------------------------------------------------------------------
#
def remove_initial_match(string, prefix):

  if initial_match(prefix, string):
    length = len(prefix)
    return string[length:]
  return string

# ----------------------------------------------------------------------------
#
def initial_match(prefix, string):

  length = len(prefix)
  return string[:length] == prefix
  
# -----------------------------------------------------------------------------
#
def initialize_shell():
  tkutil.the_dialog(python_shell_dialog)
  sparky.menu_command_callback(send_command)
  send_command('from sparky_site import *')
  send_command('from sparky_init import *')
  
# -----------------------------------------------------------------------------
#
def send_command(command):
  tkutil.the_dialog(python_shell_dialog).send_command(command)
def show_python_shell():
  tkutil.the_dialog(python_shell_dialog).show_window(1)
