# -----------------------------------------------------------------------------
# Pick peaks of a spectrum in regions near peaks of a reference spectrum.
#
import string
import Tkinter

import axes
import pyutil
import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
#
class restricted_pick_dialog(tkutil.Dialog, tkutil.Stoppable):

  def __init__(self):

    tkutil.Dialog.__init__(self, 'Restricted Peak Pick')
    
    m = sputil.view_menu(self.top, 'Find peaks in ')
    m.frame.pack(side = 'top', anchor = 'w')
    m.add_callback(self.chose_spectrum_cb)
    self.pick_menu = m

    m = sputil.spectrum_menu(self.top, 'Using selected peaks in ')
    m.frame.pack(side = 'top', anchor = 'w')
    m.add_callback(self.chose_spectrum_cb)
    self.ref_menu = m

    lbl= Tkinter.Label(self.top, text = '\nMatch axes and set tolerances')
    lbl.pack(side = 'top', anchor = 'w')
    
    f = Tkinter.Frame(self.top, borderwidth = 10, relief = 'flat')
    f.pack(side = 'top', anchor = 'w')
    self.axis_table = f

    lbl= Tkinter.Label(self.axis_table, text = self.ref_menu.get())
    lbl.grid(row = 0, column = 0, sticky = 'w')
    self.ref_label = lbl
    lbl = Tkinter.Label(self.axis_table, text = self.pick_menu.get())
    lbl.grid(row = 1, column = 0, sticky = 'w')
    self.pick_label = lbl
    lbl = Tkinter.Label(self.axis_table, text = 'Range (ppm)')
    lbl.grid(row = 2, column = 0, sticky = 'w')
    
    self.axis_labels = []
    self.range_entries = []
    self.axis_menus = []

    self.setup_axis_table()

    progress_label = Tkinter.Label(self.top, anchor = 'nw')
    progress_label.pack(side = 'top', anchor = 'w')

    br = tkutil.button_row(self.top,
			   ('Pick peaks', self.pick_cb),
                           ('Select peaks', self.select_cb),
			   ('Stop', self.stop_cb),
			   ('Close', self.close_cb),
                           ('Help', sputil.help_cb('RestrictedPick')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

    tkutil.Stoppable.__init__(self, progress_label, br.buttons[2])

  # ---------------------------------------------------------------------------
  #
  def chose_spectrum_cb(self, name):

    self.setup_axis_table()
    
  # ---------------------------------------------------------------------------
  #
  def setup_axis_table(self):

    ref_spectrum = sputil.name_to_spectrum(self.ref_menu.get())
    pick_view = sputil.name_to_view(self.pick_menu.get())
    if ref_spectrum == None or pick_view == None:
      return

    self.ref_label['text'] = ref_spectrum.name
    self.pick_label['text'] = pick_view.spectrum.name
    
    for label in self.axis_labels:
      label.destroy()
    self.axis_labels = self.create_axis_labels(ref_spectrum)

    for axis_menu in self.axis_menus:
      axis_menu.frame.destroy()
    self.axis_menus = self.create_axis_menus(ref_spectrum, pick_view.spectrum)
    
    for e in self.range_entries:
      e.frame.destroy()
    self.range_entries = self.create_range_entries(ref_spectrum)

  # ---------------------------------------------------------------------------
  #
  def create_axis_labels(self, ref_spectrum):

    axis_labels = []
    for a in range(ref_spectrum.dimension):
      axis_label = Tkinter.Label(self.axis_table)
      axis_label['text'] = 'w%d %s' % ((a+1), ref_spectrum.nuclei[a])
      axis_label.grid(row = 0, column = a+1)
      axis_labels.append(axis_label)
    return axis_labels

  # ---------------------------------------------------------------------------
  #
  def create_axis_menus(self, ref_spectrum, pick_spectrum):

    axis_menus = []
    for a in range(ref_spectrum.dimension):
      nucleus = ref_spectrum.nuclei[a]
      axis_menu = axes.axis_menu(self.axis_table, pick_spectrum, nucleus)
      axis_menu.frame.grid(row = 1, column = a+1)
      axis_menus.append(axis_menu)
    return axis_menus

  # ---------------------------------------------------------------------------
  #
  def create_range_entries(self, ref_spectrum):

    range_entries = []
    for a in range(ref_spectrum.dimension):
      if ref_spectrum.nuclei[a] == '1H':
        r = '.01'
      else:
        r = '.1'
      e = tkutil.entry_field(self.axis_table, '', r, 5)
      e.frame.grid(row = 2, column = a+1)
      range_entries.append(e)
    return range_entries
  
  # ---------------------------------------------------------------------------
  #
  def get_settings(self):

    settings = pyutil.generic_class()
    settings.ref_spectrum = sputil.name_to_spectrum(self.ref_menu.get())
    settings.pick_view = sputil.name_to_view(self.pick_menu.get())
    if settings.ref_spectrum == None or settings.pick_view == None:
      return None

    settings.ranges = []
    for e in self.range_entries:
      try:
        r = string.atof(e.variable.get())
      except ValueError:
        r = 0
      settings.ranges.append(r)
      
    settings.pick_axes = []
    for axis_menu in self.axis_menus:
      settings.pick_axes.append(axis_menu.chosen_axis())

    return settings
    
  # ---------------------------------------------------------------------------
  #
  def pick_cb(self):

    settings = self.get_settings()
    if settings:
      self.stoppable_call(self.pick_peaks, settings.pick_view,
                          settings.ref_spectrum, settings.pick_axes,
                          settings.ranges)
      message = 'Picked %d peaks in %d regions' % (self.picked,
                                                   self.region_count)
      self.progress_report(message)

  # ---------------------------------------------------------------------------
  # Use height thesholds that are the lesser of the spectrum mouse peak
  # picking thresholds and the lowest view contour levels.
  #
  def pick_peaks(self, pick_view, ref_spectrum, pick_axes, ranges):

    pick_spectrum = pick_view.spectrum

    height_thresholds = self.picking_thresholds(pick_view)
    min_linewidth = pick_spectrum.pick_minimum_linewidth
    
    ref_peaks = ref_spectrum.selected_peaks()
    self.region_count = 0
    self.picked = 0
    picked_peaks = []
    for peak in ref_peaks:

      self.region_count = self.region_count + 1
      region = self.pick_region(peak, pick_spectrum, pick_axes, ranges)
      region_peaks = pick_spectrum.pick_peaks(region,
                                              height_thresholds,
                                              min_linewidth)
      for p in region_peaks:
        p.selected = 1
      picked_peaks = picked_peaks + region_peaks
      self.picked = len(picked_peaks)
      
      message = ('%d peaks in %d of %d regions' %
                 (self.picked, self.region_count, len(ref_peaks)))
      self.progress_report(message)

  # ---------------------------------------------------------------------------
  # Take the higher of the spectrum peak picking threshold and the lowest
  # displayed contour level.
  #
  def picking_thresholds(self, pick_view):

    levels = pick_view.contour_levels
    neg_ht, pos_ht = pick_view.spectrum.pick_height_thresholds
    return (min(neg_ht, levels.negative_lowest),
            max(pos_ht, levels.positive_lowest))

  # ---------------------------------------------------------------------------
  #
  def pick_region(self, ref_peak, pick_spectrum, pick_axes, ranges):

    rmin = list(pick_spectrum.region[0])
    rmax = list(pick_spectrum.region[1])
    for a in range(len(pick_axes)):
      pick_axis = pick_axes[a]
      ref_freq = sputil.alias_axis_onto_spectrum(ref_peak.frequency[a],
                                                 pick_axis, pick_spectrum)
      rmin[pick_axis] = ref_freq - ranges[a]
      rmax[pick_axis] = ref_freq + ranges[a]
    return (tuple(rmin), tuple(rmax))
  
  # ---------------------------------------------------------------------------
  #
  def select_cb(self):

    settings = self.get_settings()
    if settings:
      self.select_peaks(settings.pick_view.spectrum, settings.ref_spectrum,
                        settings.pick_axes, settings.ranges)

  # ---------------------------------------------------------------------------
  #
  def select_peaks(self, pick_spectrum, ref_spectrum, pick_axes, ranges):

    #
    # Put reference peaks in bins
    #
    ref_peaks = ref_spectrum.selected_peaks()
    c = pyutil.cluster(ranges)
    for peak in ref_peaks:
      c.add_point(peak.frequency, peak)

    #
    # Find target peaks that are close to a binned peak
    #
    selected_peaks = []
    target_peaks = pick_spectrum.peak_list()
    for peak in target_peaks:
      p = []
      for ref_axis in range(len(pick_axes)):
        pick_axis = pick_axes[ref_axis]
        pick_freq = peak.frequency[pick_axis]
        ref_freq = sputil.alias_axis_onto_spectrum(pick_freq,
                                                   ref_axis, ref_spectrum)
        p.append(ref_freq)
      if c.has_nearby_point(p):
        selected_peaks.append(peak)

    #
    # Select the peaks
    #
    if selected_peaks:
      sparky.unselect_all_ornaments()
    for p in selected_peaks:
      p.selected = 1

    message = 'Selected %d peaks in %d regions' % (len(selected_peaks),
                                                   len(ref_peaks))
    self.progress_report(message)

# -----------------------------------------------------------------------------
#
def show_dialog():
  tkutil.the_dialog(restricted_pick_dialog).show_window(1)
