# -----------------------------------------------------------------------------
# Fork a subprocess with bidirectional i/o.
#
import os
import sys

# -----------------------------------------------------------------------------
#
class subprocess:

  def __init__(self, *argv):

    to_read, to_write = os.pipe()            # parent to subrocess pipe
    from_read, from_write = os.pipe()        # subprocess to parent pipe
  
    pid = os.fork()

    if pid == 0:

      # this is the subprocess      

      os.close(to_write)
      os.close(from_read)
      os.dup2(to_read, 0)           # connect stdin
      os.dup2(from_write, 1)        # connect stdout
      try:
        os.execvp(argv[0], argv)        # doesn't return
      except os.error, e:
        #
        # The error message can't go to the Sparky Python shell window
        # because this causes errors from the parent and child using the
        # same X connection.  So I send the error to the real stderr
        # (ie the terminal where Sparky was started.
        #
        arg_string = reduce(lambda args, arg: args + ' ' + arg, argv, '')
        sys.raw_stderr.write('Error: exec %s\n' % arg_string +
                             '  errno %d: %s\n' % (e[0], e[1]))
        #
        # Call exit with no cleanup
        #
        os._exit(1)

    # this is the parent process

    os.close(to_read)
    os.close(from_write)

    self.to_process = os.fdopen(to_write, 'w')
    self.from_process = os.fdopen(from_read, 'r');
    self.process_id = pid
    self.exit_status = None

  # ---------------------------------------------------------------------------
  #
  def __del__(self):

    self.to_process.close()
    self.from_process.close()
    if self.exit_status == None:
      os.waitpid(self.process_id, os.WNOHANG)
      # if the subprocess has not exitted it is not waitted for

  # ---------------------------------------------------------------------------
  #
  def kill(self):

    try:
      os.kill(self.process_id, 9)
    except os.error:
      pass              # process already dead
    
  # ---------------------------------------------------------------------------
  #
  def exitted(self):

    self.wait(block = 0)
    return self.exit_status != None
    
  # ---------------------------------------------------------------------------
  #
  def exit_code(self):

    self.wait(block = 1)
    return self.exit_status
    
  # ---------------------------------------------------------------------------
  #
  def wait(self, block):

    if self.exit_status == None:
      if block:
        flags = 0
      else:
        flags = os.WNOHANG
      pid, status = os.waitpid(self.process_id, flags)
      if pid == self.process_id:
        self.exit_status = status
